<?php

namespace Drupal\jsonapi_hypermedia;

use Drupal\Core\Access\AccessResultInterface;
use Drupal\Core\Cache\CacheableDependencyInterface;
use Drupal\Core\Cache\CacheableDependencyTrait;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Url;
use Drupal\jsonapi\JsonApiResource\Link as DecoratedLink;

/**
 * Decorates a JSON:API link to consider link accessibility.
 */
final class AccessRestrictedLink implements CacheableDependencyInterface {

  use CacheableDependencyTrait;

  /**
   * The access result for the link, if one was provided.
   *
   * @var \Drupal\Core\Access\AccessResultInterface|null
   */
  protected $accessResult;

  /**
   * The shimmed JSON:API link. NULL if the link is not accessible.
   *
   * @var \Drupal\jsonapi\JsonApiResource\Link
   */
  protected $inner;

  /**
   * AccessRestrictedLink constructor.
   *
   * @param \Drupal\Core\Access\AccessResultInterface $access_result
   * @param \Drupal\Core\Cache\CacheableMetadata $cacheability
   * @param \Drupal\Core\Url $url
   * @param array $link_relation_types
   * @param array $target_attributes
   */
  public function __construct(AccessResultInterface $access_result, CacheableMetadata $cacheability, Url $url, array $link_relation_types, array $target_attributes = []) {
    $this->setCacheability(CacheableMetadata::createFromObject($access_result));
    $this->accessResult = $access_result;
    $this->inner = new DecoratedLink($cacheability, $url, $link_relation_types, $target_attributes);
  }

  /**
   * Whether the link is allowed or not.
   *
   * @return bool
   *   TRUE if the link is accessible, FALSE otherwise.
   */
  public function isAllowed() {
    return $this->accessResult->isAllowed();
  }

  /**
   * Gets the bare link.
   *
   * This method should not be called unless the link is accessible.
   *
   * @return \Drupal\jsonapi\JsonApiResource\Link|null
   *   The JSON:API link.
   */
  public function getInnerLink(): DecoratedLink {
    if (!$this->isAllowed()) {
      throw new \LogicException('The shimmed link is not accessible.');
    }
    return $this->inner;
  }

}
