<?php

namespace Drupal\jsonapi_hypermedia\Annotation;

use Drupal\Component\Annotation\Plugin;

/**
 * Defines a JsonapiHypermediaProvider annotation.
 *
 * @Annotation
 */
class JsonapiHypermediaLinkProvider extends Plugin {

  /**
   * The link object key for the provided link.
   *
   * This will be the links object member name for the link returned by the
   * annotated plugin.
   *
   * @var string
   */
  public $link_key;

  /**
   * The context or contexts in which a link provider should be executed.
   *
   * The value should be a single-valued array with one of the keys documented
   * below.
   *
   * Valid array keys are 'top_level', 'resource_object' or
   * 'relationship_object'.
   *
   * Each key has a specific set of allowed values:
   *
   * 'top_level' => true|false|string
   *    TRUE if the link applies to every top-level links object, FALSE if it
   *    never applies or a type of top-level object. Possible types are:
   *      - entrypoint
   *      - success
   *      - error
   *      - individual
   *      - collection
   *      - relationship
   *
   * 'resource_object' => true|false|string
   *    TRUE if the link applies to every resource object, FALSE if it never
   *    applies or a resource type name if the link only applies to a specific
   *    type of resource object.
   *
   * 'relationship_object' => true|false|array
   *    TRUE if the link applies to every relationship object, FALSE if it never
   *    applies or a two-tuple of a resource type name and relationship field
   *    name.
   *
   * Examples:
   *
   * @JsonapiHypermediaLinkProvider(
   *   link_context={'top_level' = 'collection'}
   * )
   *
   * @JsonapiHypermediaLinkProvider(
   *   link_context={'resource_object' = 'node--page'}
   * )
   *
   * @JsonapiHypermediaLinkProvider(
   *   link_context={'relationship_object' = {'node--article', 'uid'}}
   * )
   *
   * @var array
   */
  public $link_context;

}
