<?php

namespace Drupal\jsonapi_hypermedia;

use Drupal\Core\Access\AccessResultInterface;
use Drupal\Core\Cache\CacheableDependencyInterface;
use Drupal\Core\Cache\CacheableDependencyTrait;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\jsonapi\JsonApiResource\Link as DecoratedLink;

/**
 * Decorates a JSON:API link to consider link accessibility.
 */
final class InaccessibleLink implements CacheableDependencyInterface {

  use CacheableDependencyTrait;

  /**
   * AccessRestrictedLink constructor.
   *
   * @param \Drupal\Core\Access\AccessResultInterface $access_result
   */
  public function __construct(AccessResultInterface $access_result) {
    assert(!$access_result->isAllowed());
    $this->setCacheability(CacheableMetadata::createFromObject($access_result));
  }

  /**
   * Whether the link is allowed or not.
   *
   * @return bool
   *   TRUE if the link is accessible, FALSE otherwise.
   */
  public function isAllowed() {
    return FALSE;
  }

  /**
   * Gets the bare link.
   *
   * This method should not be called unless the link is accessible.
   *
   * @return \Drupal\jsonapi\JsonApiResource\Link|null
   *   The JSON:API link.
   */
  public function getInnerLink(): DecoratedLink {
    throw new \LogicException('The shimmed link is not accessible.');
  }

}
