<?php

namespace Drupal\jsonapi_hypermedia\Plugin\jsonapi_hypermedia\LinkProvider;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Entity\EntityPublishedInterface;
use Drupal\jsonapi\JsonApiResource\ResourceObject;
use Drupal\jsonapi_hypermedia\AccessRestrictedLink;
use Drupal\jsonapi_hypermedia\LinkProviderBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class EntityPublishedLinkProvider.
 *
 * @JsonapiHypermediaLinkProvider(
 *   id = "jsonapi_hypermedia.entity_published",
 *   deriver = "Drupal\jsonapi_hypermedia\Plugin\Derivative\EntityPublishedLinkProvider",
 * )
 *
 * @internal
 */
class EntityPublishedLinkProvider extends LinkProviderBase {

  use ResourceObjectEntityLoaderTrait;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    assert(!empty($plugin_definition['published_entity_key']) && is_string($plugin_definition['published_entity_key']), "The published_entity_key must be set.");
    assert(in_array($plugin_definition['operation'], ['publish', 'unpublish'], TRUE), "The operation must be set to either 'publish' or 'unpublish'.");
    $provider = new static($configuration, $plugin_id, $plugin_definition);
    $provider->setEntityTypeManager($container->get('entity_type.manager'));
    return $provider;
  }

  /**
   * {@inheritdoc}
   */
  public function getLink($resource_object) {
    assert($resource_object instanceof ResourceObject);
    $entity = $this->loadEntityFromResourceObject($resource_object);
    assert($entity instanceof EntityPublishedInterface);
    $plugin_definition = $this->getPluginDefinition();
    $published_key = $plugin_definition['published_entity_key'];
    $published = $entity->isPublished();
    $access_result = AccessResult::allowedIf($plugin_definition['operation'] === 'publish' XOR $published)
      ->andIf($entity->access('update', NULL, TRUE))
      ->andIf($entity->{$published_key}->access('edit', NULL, TRUE))
      ->addCacheableDependency($entity);
    $link_relations = ['https://jsonapi.org/profiles/drupal/hypermedia/#update'];
    $link_attributes = [
      'data' => [
        'type' => $resource_object->getTypeName(),
        'id' => $resource_object->getId(),
        'attributes' => [
          $published_key => (int) !$published,
        ]
      ],
    ];
    return new AccessRestrictedLink($access_result, CacheableMetadata::createFromObject($resource_object), $resource_object->toUrl(), $link_relations, $link_attributes);
  }

}
