<?php

namespace Drupal\jsonapi_hypermedia\Plugin\jsonapi_hypermedia\LinkProvider;

use Drupal\Core\Cache\CacheableMetadata;
use Drupal\jsonapi\JsonApiResource\ResourceObject;
use Drupal\jsonapi_hypermedia\AccessRestrictedLink;
use Drupal\jsonapi_hypermedia\LinkProviderBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class EntityUpdateDeleteLinkProvider.
 *
 * @JsonapiHypermediaLinkProvider(
 *   id = "jsonapi_hypermedia.entity_update_delete",
 *   deriver = "Drupal\jsonapi_hypermedia\Plugin\Derivative\EntityUpdateDeleteLinkProvider",
 * )
 *
 * @internal
 */
class EntityUpdateDeleteLinkProvider extends LinkProviderBase {

  use ResourceObjectEntityLoaderTrait;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    assert(in_array($plugin_definition['operation'], ['update', 'delete'], TRUE), "The operation must be set to either 'update' or 'delete'.");
    $provider = new static($configuration, $plugin_id, $plugin_definition);
    $provider->setEntityTypeManager($container->get('entity_type.manager'));
    return $provider;
  }

  public function getLink($resource_object) {
    assert($resource_object instanceof ResourceObject);
    $plugin_definition = $this->getPluginDefinition();
    $entity = $this->loadEntityFromResourceObject($resource_object);
    $link_relations = [
      'self',
      'https://jsonapi.org/profiles/drupal/hypermedia/#' . $plugin_definition['operation'],
    ];
    $access = $entity->access($plugin_definition['operation'], NULL, TRUE);
    return new AccessRestrictedLink($access, new CacheableMetadata(), $resource_object->toUrl(), $link_relations);
  }
}
